/*
 Soekris Netxxxx GPIO driver
 Common functions shared between the net4501 and net4801 driver

  (c) Copyright 2003-2004   Martin Hejl <martin@hejl.de>
                            G&H Softwareentwicklung GmbH

 **********************************************************************
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 **********************************************************************
 *
 */
 /* Deal with CONFIG_MODVERSIONS */

#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
	#include <linux/modversions.h>
#endif

#define TEMP_PROC_FILE 8
#define VOLT_PROC_FILE 16

#include "common.h"

struct gpio_operations *driver_ops;
struct semaphore gpio_sema;
static unsigned long init_map;

extern int net4801_init(struct gpio_operations **driver_ops);
void __exit common_cleanup(void);

int toString(unsigned long value, char* buffer, int number_of_bits) {
	static int i;

	/* convert it into a string */
	for(i=number_of_bits;i>0;i--){
		buffer[number_of_bits-i]=test_bit(i-1,&value)?'1':'0';
	}

	buffer[number_of_bits] = '\n';
	buffer[number_of_bits+1] = 0;

	return number_of_bits+1;
}

unsigned long  fromString(char* buffer, int number_of_bits) {
	static int i;
	static unsigned long ret_val=0;

	ret_val = 0;
	/* Create WORD to write from the string */
	for(i=0;i<number_of_bits;i++){
		if (buffer[i] == 0) break;

		if (buffer[i] == '0' || buffer[i] == '1') {
			ret_val=ret_val<<1;
		}

		if (buffer[i] == '1') {
			ret_val |= 1;
		}
	}
	return(ret_val);
}


//------------------------------------------------

// PROC file

static int procfile_temperature_read(
                               char *buffer,
                               __attribute__ ((unused)) char **start,
                               off_t offset,
                               int buffer_length,
                               int *eof,
                               __attribute__ ((unused)) void *data)
{
	int len;

	if (driver_ops->readTemperature==NULL ||
                               offset > 0 ||
                               buffer_length<TEMPERATURE_BUFFER_SIZE)
	{
		return 0;
	}

	if (down_interruptible(&gpio_sema))
		return -ERESTARTSYS;

	MOD_INC_USE_COUNT;

	len = driver_ops->readTemperature(buffer, buffer_length);

	*eof=1;

	MOD_DEC_USE_COUNT;
	up(&gpio_sema);
	return len;
}

static int procfile_voltage_read(
                               char *buffer,
                               __attribute__ ((unused)) char **start,
                               off_t offset,
                               int buffer_length,
                               int *eof,
                               __attribute__ ((unused)) void *data)
{
	int len;

	if (driver_ops->readVoltage==NULL ||
          offset > 0 ||
          buffer_length<VOLTAGE_BUFFER_SIZE)
	{
		return 0;
	}

	if (down_interruptible(&gpio_sema))
		return -ERESTARTSYS;

	MOD_INC_USE_COUNT;

	len = driver_ops->readVoltage(buffer, buffer_length);

	*eof=1;

	MOD_DEC_USE_COUNT;
	up(&gpio_sema);

	return len;
}

//------------------------------------------------

int __init common_init(void)
{
	static int result;
	static struct proc_dir_entry *Temperature_Proc_File;
	static struct proc_dir_entry *Voltage_Proc_File;

	init_map = 0;
	result = net4801_init(&driver_ops);
	if (result!=0) {
		return(result);
	}

	if (driver_ops->readTemperature!=NULL) {
		Temperature_Proc_File = create_proc_entry(
				TEMPERATURE_PROC_FILENAME,
				S_IRUSR | S_IRGRP | S_IROTH,
				NULL);

		if(Temperature_Proc_File == NULL) {
			printk(KERN_ERR OUR_NAME ": Could not register " TEMPERATURE_PROC_FILENAME ". Terminating\n");
			common_cleanup();
			return -ENOMEM;
		} else {
			Temperature_Proc_File->read_proc=procfile_temperature_read;
			init_map |= TEMP_PROC_FILE;
		}
	}

	if (driver_ops->readVoltage!=NULL) {
		Voltage_Proc_File = create_proc_entry(
				VOLTAGE_PROC_FILENAME,
				S_IRUSR | S_IRGRP | S_IROTH,
				NULL);

		if(Voltage_Proc_File  == NULL) {
			printk(KERN_ERR OUR_NAME ": Could not register " VOLTAGE_PROC_FILENAME ". Terminating\n");
			common_cleanup();
			return -ENOMEM;
		} else {
			Voltage_Proc_File->read_proc=procfile_voltage_read;
			init_map |= VOLT_PROC_FILE;
		}
	}

	sema_init(&gpio_sema, 1);

	return(0);
}

void __exit common_cleanup(void)
{
	if (init_map&VOLT_PROC_FILE)
		remove_proc_entry(VOLTAGE_PROC_FILENAME, NULL);

	if (init_map&TEMP_PROC_FILE)
		remove_proc_entry(TEMPERATURE_PROC_FILENAME, NULL);

	driver_ops->cleanup();
}

//------------------------------------------------

module_init(common_init);
module_exit(common_cleanup);

MODULE_AUTHOR("Martin Hejl");
MODULE_DESCRIPTION("Soekris net5xxx env driver");
MODULE_LICENSE("GPL");
EXPORT_NO_SYMBOLS;
